;;;; -*- Mode: LISP; Syntax: ANSI-Common-Lisp; Base: 10 -*-
;;;;
;;;; CMUCL 19c Patch 002: Fix a bug in type derivation in computing
;;;; the union two intervals.  (See mail to cmucl-imp by Mika
;;;; Pihlajamaki, 28 Dec 2005.  The following code illustrates the bug
;;;;
;;;; (defun foo ()
;;;;  (* 0
;;;;     (+ (if (< 1 1)
;;;;	    1
;;;;	    1)
;;;;	3.14)))
;;;;
;;;; Compiling and running this functions causes an error instead of
;;;; returning 0.0.
;;;;
;;;; It turns out that this bug also exists in 19b and earlier.  I
;;;; think this patch will also work in earlier versions, but I did
;;;; not check that.

(in-package "C")

#+(and :cmu19 (not cmucl-19c-patch-002))
(ext:without-package-locks
(defun interval-merge-pair (x y)
  (declare (type interval x y))
  ;; If x and y intersect or are adjacent, create the union.
  ;; Otherwise return nil
  (when (or (interval-intersect-p x y)
	    (interval-adjacent-p x y))
    (flet ((select-bound (x1 x2 min-op max-op)
	     (let ((x1-val (bound-value x1))
		   (x2-val (bound-value x2)))
	       (cond ((and x1 x2)
		      ;; Both bounds are finite.  Select the right one.
		      (cond ((funcall min-op x1-val x2-val)
			     ;; x1 definitely better
			     x1)
			    ((funcall max-op x1-val x2-val)
			     ;; x2 definitely better
			     x2)
			    (t
			     ;; Bounds are equal.  Select either value and
			     ;; make it open only if both were open.
			     (set-bound x1-val (and (consp x1) (consp x2))))))
		     (t
		      ;; At least one bound is not finite.  The non-finite
		      ;; bound always wins.
		      nil)))))
      (let* ((x-lo (copy-interval-limit (interval-low x)))
	     (x-hi (copy-interval-limit (interval-high x)))
	     (y-lo (copy-interval-limit (interval-low y)))
	     (y-hi (copy-interval-limit (interval-high y))))
	(make-interval :low (select-bound x-lo y-lo
					  #'signed-zero-< #'signed-zero->)
		       :high (select-bound x-hi y-hi
					   #'signed-zero-> #'signed-zero-<))))))  
)

(pushnew :cmucl-19c-patch-002 *features*)
